import { world, system, ItemStack } from "@minecraft/server";
import { ActionFormData } from '@minecraft/server-ui';

world.beforeEvents.worldInitialize.subscribe((initEvent) => {
  const registryBlock = initEvent.blockComponentRegistry;
  const registryItem = initEvent.itemComponentRegistry;
  registryItem.registerCustomComponent("db:dragon_radar", {
    onUse: dragon_radar,
  });
  registryItem.registerCustomComponent("db:dragon_ball", {
    onUseOn: dragon_ball,
  });
  registryBlock.registerCustomComponent("db:dragon_ball_actived", {
    onPlayerInteract: dragon_ball_actived,
  });
  registryBlock.registerCustomComponent("db:dragon_ball_break", {
    onPlayerDestroy: dragon_ball_break,
  });
  registryBlock.registerCustomComponent("db:dragon_ball_shine", {
    onTick: dragon_ball_shine,
  });
})

system.runInterval(() => {
  let stonesActive = world.getDynamicProperty("db:dragonball_stones");
  let stonesStartTime = world.getDynamicProperty("db:dragonball_stones_time");
  let currentTime = Date.now();

  if (stonesActive) {
    if (stonesStartTime && currentTime - stonesStartTime >= 30 * 60 * 1000) {
      world.setDynamicProperty("db:dragonball_stones", false);
      world.setDynamicProperty("db:dragonball_stones_time", null);
    } else {

      return;
    }
  }

  world.getAllPlayers().forEach(player => {
    let playerX = Math.floor(player.location.x);
    let playerZ = Math.floor(player.location.z);

    for (let i = 1; i <= 7; i++) {
      let ballKey = `db:dragonball_set_${i}`;
      let locationKey = `db:dragonball_${i}_location`;

      let dragonballFound = world.getDynamicProperty(ballKey);
      let dragonballLocation = world.getDynamicProperty(locationKey);

      if (!dragonballFound && dragonballLocation) {
        let { x, z } = dragonballLocation;
        let distanceX = Math.abs(playerX - x);
        let distanceZ = Math.abs(playerZ - z);

        if (distanceX > 40 || distanceZ > 40) continue;

        for (let y = 60; y <= 319; y++) {
          try {
            let position = { x, y, z };
            let belowBlock = player.dimension.getBlock({ x, y: y - 1, z });
            let currentBlock = player.dimension.getBlock(position);
            let aboveBlock = player.dimension.getBlock({ x, y: y + 1, z });

            if (belowBlock && belowBlock.typeId !== "minecraft:air" &&
              currentBlock && currentBlock.typeId === "minecraft:air" &&
              aboveBlock && aboveBlock.typeId === "minecraft:air") {

              let starNumber = i;
              player.dimension.setBlockType(position, `db:dragon_balls_${starNumber}`);
              world.setDynamicProperty(ballKey, true);
              return;
            }
          } catch (error) { }
        }
      }
    }
  });
});



function dragon_radar(event) {
  const { source } = event;
  let stonesActive = world.getDynamicProperty("db:dragonball_stones");
  let stonesStartTime = world.getDynamicProperty("db:dragonball_stones_time");

  if (stonesActive) {
    if (stonesStartTime) {
      let currentTime = Date.now();
      let elapsed = currentTime - stonesStartTime;
      let remaining = 30 * 60 * 1000 - elapsed;
      let minutes = Math.floor(remaining / (60 * 1000));
      let seconds = Math.floor((remaining % (60 * 1000)) / 1000);
      
      source.onScreenDisplay.setActionBar({
        translate: "stone.dragonballs.timer",
        with: [minutes.toString(), seconds.toString()]
      });
    } else {
      source.onScreenDisplay.setActionBar({
        translate: "stone.dragonballs.ad"
      });
    }
    return;
  }


  for (let i = 1; i <= 7; i++) {
    let ballKey = `db:dragonball_${i}`;
    let locationKey = `db:dragonball_${i}_location`;

    let dragonballFound = world.getDynamicProperty(ballKey);
    let dragonballLocation = world.getDynamicProperty(locationKey);

    if (!dragonballFound) {
      if (!dragonballLocation || dragonballLocation === null) {
        let randomX = Math.floor(Math.random() * 1000) - 500;
        let randomZ = Math.floor(Math.random() * 1000) - 500;
        let newLocation = { x: randomX, y: 0, z: randomZ };

        world.setDynamicProperty(locationKey, newLocation);
        source.onScreenDisplay.setActionBar({
          translate: `create.dragonball.${i}`
        });
        return;
      }

      let dx = source.location.x - dragonballLocation.x;
      let dz = source.location.z - dragonballLocation.z;
      let distance = Math.sqrt(dx * dx + dz * dz);

      if (world.getDynamicProperty(ballKey)) {
        source.onScreenDisplay.setActionBar({
          translate: `congratulations.dragonball.found_${i}`
        });
      } else {
        let diffX = Math.floor(Math.abs(source.location.x - dragonballLocation.x));
        let diffZ = Math.floor(Math.abs(source.location.z - dragonballLocation.z));

        source.onScreenDisplay.setActionBar({
          translate: `location.dragonball.${i}`,
          with: [`${diffX}`, `${diffZ}`]
        });
      }
      return;
    } 4
  }

  source.onScreenDisplay.setActionBar({
    translate: `congratulations.dragonball.all`
  });
}

function dragon_ball_shine(event) {
  const { dimension, block } = event;
  try {
    const dragonball_place = block.permutation.getState(`db:shine`);
    dimension.spawnParticle('db:shine', block.center());
    if (!dragonball_place) {
      block.setPermutation(block.permutation.withState(`db:shine`, true));
    } else if (dragonball_place) {
      block.setPermutation(block.permutation.withState(`db:shine`, false));
    }
    if (!block.permutation.getState(`db:player`)) {
      const players = dimension.getPlayers({
        maxDistance: 12,
        closest: 1,
        location: block.location
      });

      if (players.length > 0) {
        players.forEach(player => {
          block.setPermutation(block.permutation.withState(`db:player`, true));
          system.runTimeout(() => {

            wish(player, block);
          }, 80);
        })
      }
    }
  } catch (error) {
  }
}

function wish(player, block) {
  const form = new ActionFormData()
    .title({ translate: 'wish.body.title' })
    .body({ translate: 'wish.body.choose' })
    .button({ translate: 'wish.items.diamond' }, 'textures/items/diamond')
    .button({ translate: 'wish.items.emerald' }, 'textures/items/emerald')
    .button({ translate: 'wish.items.netherite' }, 'textures/items/netherite_ingot')
    .button({ translate: 'wish.items.gold' }, 'textures/items/gold_ingot')
    .button({ translate: 'wish.items.iron' }, 'textures/items/iron_ingot')
    .button({ translate: 'wish.items.copper' }, 'textures/items/copper_ingot')
    .button({ translate: 'wish.items.quartz' }, 'textures/items/quartz')
    .button({ translate: 'wish.items.blaze_rod' }, 'textures/items/blaze_rod')
    .button({ translate: 'wish.items.ender_pearl' }, 'textures/items/ender_pearl')
    .button({ translate: 'wish.items.ender_eye' }, 'textures/items/ender_eye')
    .button({ translate: 'wish.items.wither_skull' }, 'textures/items/skull_wither')
    .button({ translate: 'wish.items.trident' }, 'textures/items/trident')
    .button({ translate: 'wish.items.heart_of_the_sea' }, 'textures/items/heartofthesea_closed')
    .button({ translate: 'wish.items.nautilus' }, 'textures/items/nautilus')
    .button({ translate: 'wish.items.coal' }, 'textures/items/coal')
    .button({ translate: 'wish.items.redstone' }, 'textures/items/redstone_dust')
    .button({ translate: 'wish.items.amethyst' }, 'textures/items/amethyst_shard')
    .show(player)
    .then(response => {
      if (response.canceled) {
        wish(player, block);
        return;
      }

      let itemID, amount;

      switch (response.selection) {
        case 0: // 8 Diamantes
          itemID = 'minecraft:diamond';
          amount = 8;
          break;
        case 1: // 8 Esmeraldas
          itemID = 'minecraft:emerald';
          amount = 8;
          break;
        case 2: // 1 Lingote de Netherite
          itemID = 'minecraft:netherite_ingot';
          amount = 1;
          break;
        case 3: // 32 Oro (movido arriba)
          itemID = 'minecraft:gold_ingot';
          amount = 32;
          break;
        case 4: // 28 Hierro
          itemID = 'minecraft:iron_ingot';
          amount = 28;
          break;
        case 5: // 34 Cobre
          itemID = 'minecraft:copper_ingot';
          amount = 34;
          break;
        case 6: // 8 Cuarzo
          itemID = 'minecraft:quartz';
          amount = 8;
          break;
        case 7: // 1 Vara de Blaze
          itemID = 'minecraft:blaze_rod';
          amount = 1;
          break;
        case 8: // 3 Perlas de Ender
          itemID = 'minecraft:ender_pearl';
          amount = 3;
          break;
        case 9: // 4 Ojos de Ender
          itemID = 'minecraft:ender_eye';
          amount = 4;
          break;
        case 10: // 1 Cabeza de Wither
          itemID = 'minecraft:wither_skeleton_skull';
          amount = 1;
          break;
        case 11: // 1 Tridente
          itemID = 'minecraft:trident';
          amount = 1;
          break;
        case 12: // 1 Corazón del Mar
          itemID = 'minecraft:heart_of_the_sea';
          amount = 1;
          break;
        case 13: // 4 Conchas de Nautilus
          itemID = 'minecraft:nautilus_shell';
          amount = 4;
          break;
        case 14: // 34 Carbón
          itemID = 'minecraft:coal';
          amount = 34;
          break;
        case 15: // 40 Redstone
          itemID = 'minecraft:redstone';
          amount = 40;
          break;
        case 16: // 32 Amatista
          itemID = 'minecraft:amethyst_shard';
          amount = 32;
          break;
        default:
          wish(player, block);
          return;
      }
      try {
        const wishItem = new ItemStack(itemID, amount);
        player.dimension.spawnItem(wishItem, block.location);
        const dragonball = player.dimension.getBlock(block.location);
        player.playSound('mob.shelong.wish');
        const shelongs = player.dimension.getEntities({
          maxDistance: 1,
          closest: 1,
          location: block.location,
          families: ["shelong"],
        });

        if (shelongs.length > 0) {
          shelongs.forEach(shelong => {
            player.playSound('mob.shelong.stone');
            player.dimension.spawnEntity(`db:dragon_ball_stones`, {
              x: block.location.x + 0.5,
              y: block.location.y,
              z: block.location.z + 0.5
            });
            shelong.triggerEvent("despawn");
            player.dimension.setBlockType(dragonball, `minecraft:air`);
            player.runCommand("time set noon");
          });
        }
      } catch (error) {
        wish(player, block);
      }
    });
}


function dragon_ball_actived(event) {
  const { block, player, dimension } = event;
  const inventory = player.getComponent("minecraft:inventory");
  const item = inventory.container.getItem(player.selectedSlotIndex);

  let countDragonBall = 0;
  let cardinalDirection = block.permutation.getState("minecraft:cardinal_direction");
  for (let i = 1; i <= 7; i++) {
    const dragonball_place = block.permutation.getState(`db:dragonball_place_${i}`);
    if (dragonball_place) {
      countDragonBall++;
    }
  }

  if (countDragonBall === 7) {
    for (let i = 1; i <= 7; i++) {
      let ballKey = `db:dragonball_${i}`;
      let ballKey2 = `db:dragonball_set_${i}`;
      let locationKey = `db:dragonball_${i}_location`;
      world.setDynamicProperty(ballKey, false);
      world.setDynamicProperty(ballKey2, false);
      world.setDynamicProperty(locationKey, null);

    }
    world.setDynamicProperty("db:dragonball_stones", true);
    world.setDynamicProperty("db:dragonball_stones_time", Date.now());
    player.runCommand("time set midnight")
    dimension.setBlockType(block, `db:dragon_ball_shelong`);
    const dragonball = dimension.getBlock(block.location);


    dragonball.setPermutation(dragonball.permutation.withState(`minecraft:cardinal_direction`, cardinalDirection));
    player.playSound('mob.shelong.spawn');
    const shelong = dimension.spawnEntity(`db:shelong_${cardinalDirection}`, { x: block.location.x + 0.5, y: block.location.y, z: block.location.z + 0.5 });
    dimension.spawnParticle('db:shine_large', block.center());
    return;
  }

  if (!item) return;
  const match = item.typeId.match(/^db:dragon_ball_(\d+)_star$/);
  if (!match) return;
  const starNumber = match[1];
  const dragonball_place = block.permutation.getState(`db:dragonball_place_${starNumber}`);
  if (!dragonball_place) {
    inventory.container.setItem(player.selectedSlotIndex, null);
    block.setPermutation(block.permutation.withState(`db:dragonball_place_${starNumber}`, true));
  }
}

function dragon_ball_break(event) {
  const { block, destroyedBlockPermutation, player, dimension } = event;

  for (let i = 1; i <= 7; i++) {
    const dragonball_place = destroyedBlockPermutation.getState(`db:dragonball_place_${i}`);
    if (dragonball_place) {
      const dragonBall = new ItemStack(`db:dragon_ball_${i}_star`, 1);
      dimension.spawnItem(dragonBall, block.location);
    }
  }

  if (destroyedBlockPermutation.type.id.startsWith("db:dragon_balls")) {
    const match = destroyedBlockPermutation.type.id.match(/^db:dragon_balls_(\d+)$/);
    if (match) {
      const ballKey = `db:dragonball_${match[1]}`;
      world.setDynamicProperty(ballKey, true);
    }
  }
}



function dragon_ball(event) {
  const { block, itemStack: item, source } = event;
  const inventory = source.getComponent("minecraft:inventory");
  const blockPos = block.location;
  const blockBelowPos = { x: blockPos.x, y: blockPos.y + 1, z: blockPos.z };
  const blockBelow = source.dimension.getBlock(blockBelowPos);

  const match = item.typeId.match(/^db:dragon_ball_(\d+)_star$/);
  if (!match || block.typeId === "minecraft:air") return;

  const starNumber = match[1];

  if (blockBelow.typeId === "minecraft:air" && !block.typeId.startsWith("db:dragon_ball")) {
    source.dimension.setBlockType(blockBelowPos, `db:dragon_ball_${starNumber}_star`);
    const dragonball = source.dimension.getBlock(blockBelowPos);

    const playerRotation = source.getRotation().y;

    let direction = "north";
    if (playerRotation >= -45 && playerRotation < 45) {
      direction = "south";
    } else if (playerRotation >= 45 && playerRotation < 135) {
      direction = "west";
    } else if (playerRotation >= -135 && playerRotation < -45) {
      direction = "east";
    }
    dragonball.setPermutation(dragonball.permutation.withState(`minecraft:cardinal_direction`, direction));
    inventory.container.setItem(source.selectedSlotIndex, null);
  }

}
